static class CollisionDetector {
  static boolean rectRect(float r1x, float r1y, float r1w, float r1h, float r2x, float r2y, float r2w, float r2h) {
    r1x -= r1w*0.5;
    r1y -= r1h*0.5;
    r2x -= r2w*0.5;
    r2y -= r2h*0.5;
  
    if (r1x + r1w >= r2x &&    // r1 right edge past r2 left
        r1x <= r2x + r2w &&    // r1 left edge past r2 right
        r1y + r1h >= r2y &&    // r1 top edge past r2 bottom
        r1y <= r2y + r2h) {    // r1 bottom edge past r2 top
          return true;
    }
    return false;
  }
  
  static float floatRectRect(float r1x, float r1y, float r1w, float r1h, float r2x, float r2y, float r2w, float r2h) {
    r1x -= r1w*0.5;
    r1y -= r1h*0.5;
    r2x -= r2w*0.5;
    r2y -= r2h*0.5;
  
    if (r1x + r1w >= r2x &&    // r1 right edge past r2 left
        r1x <= r2x + r2w &&    // r1 left edge past r2 right
        r1y + r1h >= r2y &&    // r1 top edge past r2 bottom
        r1y <= r2y + r2h) {    // r1 bottom edge past r2 top
          return r1y+r1h-r2y;
    }
    return 0;
  }
  
  static boolean circleRect(float cx, float cy, float radius, float rx, float ry, float rw, float rh) {
    rx -= rw*0.5;
    ry -= rh*0.5;
  
    // temporary variables to set edges for testing
    float testX = cx;
    float testY = cy;
  
    // which edge is closest?
    if (cx < rx-rw/2)         testX = rx;      // test left edge
    else if (cx > rx+rw/2) testX = rx+rw;   // right edge
    if (cy < ry-rh/2)         testY = ry;      // top edge
    else if (cy > ry+rh/2) testY = ry+rh;   // bottom edge
  
    // get distance from closest edges
    float distX = cx-testX;
    float distY = cy-testY;
    float distance = sqrt( (distX*distX) + (distY*distY) );
  
    // if the distance is less than the radius, collision!
    if (distance <= radius) {
      return true;
    }
    return false;
  }
  
  // POLYGON/RECTANGLE
  static boolean polyRect(PVector[] vertices, float rx, float ry, float rw, float rh) {
    rx -= rw*0.5;
    ry -= rh*0.5;
  
    // go through each of the vertices, plus the next
    // vertex in the list
    int next = 0;
    for (int current=0; current<vertices.length; current++) {
  
      // get next vertex in list
      // if we've hit the end, wrap around to 0
      next = current+1;
      if (next == vertices.length) next = 0;
  
      // get the PVectors at our current position
      // this makes our if statement a little cleaner
      PVector vc = vertices[current];    // c for "current"
      PVector vn = vertices[next];       // n for "next"
  
      // check against all four sides of the rectangle
      boolean collision = lineRect(vc.x,vc.y,vn.x,vn.y, rx,ry,rw,rh);
      if (collision) return true;
  
      // optional: test if the rectangle is INSIDE the polygon
      // note that this iterates all sides of the polygon
      // again, so only use this if you need to
      boolean inside = polygonPoint(vertices, rx,ry);
      if (inside) return true;
    }
  
    return false;
  }
  
  // LINE/RECTANGLE
  static boolean lineRect(float x1, float y1, float x2, float y2, float rx, float ry, float rw, float rh) {
  
    // check if the line has hit any of the rectangle's sides
    // uses the Line/Line function below
    boolean left =   lineLine(x1,y1,x2,y2, rx,ry,rx, ry+rh);
    boolean right =  lineLine(x1,y1,x2,y2, rx+rw,ry, rx+rw,ry+rh);
    boolean top =    lineLine(x1,y1,x2,y2, rx,ry, rx+rw,ry);
    boolean bottom = lineLine(x1,y1,x2,y2, rx,ry+rh, rx+rw,ry+rh);
  
    // if ANY of the above are true,
    // the line has hit the rectangle
    if (left || right || top || bottom) {
      return true;
    }
    return false;
  }
  
  
  // LINE/LINE
  static boolean lineLine(float x1, float y1, float x2, float y2, float x3, float y3, float x4, float y4) {
  
    // calculate the direction of the lines
    float uA = ((x4-x3)*(y1-y3) - (y4-y3)*(x1-x3)) / ((y4-y3)*(x2-x1) - (x4-x3)*(y2-y1));
    float uB = ((x2-x1)*(y1-y3) - (y2-y1)*(x1-x3)) / ((y4-y3)*(x2-x1) - (x4-x3)*(y2-y1));
  
    // if uA and uB are between 0-1, lines are colliding
    if (uA >= 0 && uA <= 1 && uB >= 0 && uB <= 1) {
      return true;
    }
    return false;
  }
  
  // POLYGON/POINT
  // only needed if you're going to check if the rectangle
  // is INSIDE the polygon
  static boolean polygonPoint(PVector[] vertices, float px, float py) {
    boolean collision = false;
  
    // go through each of the vertices, plus the next
    // vertex in the list
    int next = 0;
    for (int current=0; current<vertices.length; current++) {
  
      // get next vertex in list
      // if we've hit the end, wrap around to 0
      next = current+1;
      if (next == vertices.length) next = 0;
  
      // get the PVectors at our current position
      // this makes our if statement a little cleaner
      PVector vc = vertices[current];    // c for "current"
      PVector vn = vertices[next];       // n for "next"
  
      // compare position, flip 'collision' variable
      // back and forth
      if (((vc.y > py && vn.y < py) || (vc.y < py && vn.y > py)) &&
           (px < (vn.x-vc.x)*(py-vc.y) / (vn.y-vc.y)+vc.x)) {
              collision = !collision;
      }
    }
    return collision;
  }
  
  static boolean pointRect(float px, float py, float rx, float ry, float rw, float rh) {

    // is the point inside the rectangle's bounds?
    if (px >= rx - rw/2 &&        // right of the left edge AND
        px <= rx + rw/2 &&   // left of the right edge AND
        py >= ry - rh/2 &&        // below the top AND
        py <= ry + rh/2) {   // above the bottom
          return true;
    }
    return false;
  }
}